@@ -15,7 +15,7 @@
 """Functionality for managing and executing computational graphs.
 """
 
-__version__ = "0.4.0"
+__version__ = "0.5.0"
 
 import collections
 import enum
@@ -35,48 +35,22 @@ class DeprecationWarning(Warning):
 
 
 class ExpressionTask(object):
-    """Manage a task that executes Python expressions and tracks implicit dependencies.
+    """.. deprecated:: 0.5.0
 
-    Parameters
-    ----------
-    graph: class:`Graph`, required
-        The graph where the new expression task will be created.
-    name: hashable object, required
-        Unique name for the new expression task.
-    expression: string, required
-        Python expression that will be executed whenever the task is executed.
-    locals: dict, optional
-        Optional dictionary containing local objects that will be available for
-        use in the expression.
+    Use :meth:`Graph.set_expression` instead.
     """
     def __init__(self, graph, name, expression, locals={}):
+        warnings.warn("graphcat.ExpressionTask is deprecated, use Graph.set_expression() instead.", DeprecationWarning, stacklevel=2)
         self._graph = graph
         self._name = name
         self.set(expression, locals)
 
     def set(self, expression, locals={}):
-        """Change the Python expression to be executed.
+        """.. deprecated:: 0.5.0
 
-        Parameters
-        ----------
-        expression: string, required
-            New Python expression that will be executed whenever the task is executed.
-        locals: dict, optional
-            Optional dictionary containing local objects that will be available for
-            use in the expression.
+        Use :meth:`Graph.set_expression` instead.
         """
-        self._graph.set_task(self._name, execute(expression, locals))
-
-        sources = list(self._graph._graph.successors(self._name))
-        for source in sources:
-            self._graph._graph.remove_edge(self._name, source)
-
-        updated = UpdatedTasks(self._graph)
-        self._graph.update(self._name)
-
-        sources = updated.tasks.difference([self._name])
-        for source in sources:
-            self._graph._graph.add_edge(self._name, source, input=Input.DEPENDENCY)
+        self._graph.set_expression(self._name, expression, locals)
 
 
 class Graph(object):
@@ -98,6 +72,10 @@ class Graph(object):
         self._on_update = blinker.Signal()
 
 
+    def __contains__(self, name):
+        return name in self._graph
+
+
     def _require_valid_names(self, names):
         if names is None:
             return self.tasks()
@@ -116,11 +94,6 @@ class Graph(object):
             raise ValueError(f"Task {name!r} already exists.")
 
 
-    def _require_link_present(self, source, target):
-        if self._graph.number_of_edges(target, source) != 1:
-            raise ValueError(f"Edge {source!r} -> {target!r} doesn't exist.")
-
-
     def add_links(self, source, targets):
         """Add links between `source` and `targets`.
 
@@ -186,11 +159,27 @@ class Graph(object):
 
 
     def clear_links(self, source, target):
+        """Remove links from the graph.
+
+        This method will remove all links from `source` to `target`.
+
+        Parameters
+        ----------
+        source: hashable object, required
+            Source task name.
+        target: hashable object, required
+            Target task name.
+
+        Raises
+        ------
+        :class:`ValueError`
+            If `source` or `task` don't exist.
+        """
         self._require_task_present(source)
         self._require_task_present(target)
-        self._require_link_present(source, target)
         self.mark_unfinished(source)
-        self._graph.remove_edge(target, source)
+        while self._graph.number_of_edges(target, source):
+            self._graph.remove_edge(target, source)
 
 
     def clear_tasks(self, names=None):
@@ -413,6 +402,35 @@ class Graph(object):
         return self._graph.nodes[name]["output"]
 
 
+    def set_expression(self, name, expression, locals={}):
+        """Create a task that will execute a Python expression.
+
+        The task will automatically track implicit dependencies.
+
+        Parameters
+        ----------
+        name: hashable object, required
+            Unique name for the new expression task.
+        expression: string, required
+            Python expression that will be executed whenever the task is executed.
+        locals: dict, optional
+            Optional dictionary containing local objects that will be available for
+            use in the expression.
+        """
+        self.set_task(name, execute(expression, locals))
+
+        sources = list(self._graph.successors(name))
+        for source in sources:
+            self._graph.remove_edge(name, source)
+
+        updated = UpdatedTasks(self)
+        self.update(name)
+
+        sources = updated.tasks.difference([name])
+        for source in sources:
+            self._graph.add_edge(name, source, input=Input.DEPENDENCY)
+
+
     def set_links(self, source, targets):
         """Set links between `source` and `targets`.
 
@@ -705,31 +723,6 @@ class UpdatedTasks(object):
 
 
 
-class VariableTask(object):
-    """.. deprecated:: 0.4.0
-
-    Use :meth:`set_task` or :meth:`set_parameter` instead.
-    """
-    warnings.warn("graphcat.VariableTask is deprecated, use Graph.set_task() or Graph.set_parameter() instead.", DeprecationWarning, stacklevel=2)
-    def __init__(self, graph, name, value):
-        self._graph = graph
-        self._name = name
-        self._graph.add_task(name, constant(value))
-
-    def set(self, value):
-        """Change the underlying task value.
-
-        This will cause downstream tasks to execute the next time they're
-        updated.
-
-        Parameters
-        ----------
-        value: Any value, required
-            New value of the variable.
-        """
-        self._graph.set_task(self._name, constant(value))
-
-
 def constant(value):
     """Factory for task functions that return constant values when executed.
 
@@ -794,6 +787,28 @@ def null(name, inputs):
     pass
 
 
+def passthrough(input, index=0):
+    """Factory for task functions that pass-through incoming data.
+
+    Callers can use this function to temporarily bypass tasks in the graph.
+
+    Parameters
+    ----------
+    input: hashable object, required
+        The named input that will pass-through to the task output.
+    index: integer, required
+        Index of the named inputs that will pass-through to the task output.
+
+    Returns
+    -------
+    fn: function
+        Task function that will pass input `input` index `index` to its output.
+    """
+    def implementation(name, inputs):
+        return inputs[input][index]
+    return implementation
+
+
 def raise_exception(exception):
     """Factory for task functions that raise an exception when executed.
 
